const config = require('config')
const { chain, get, isEmpty } = require('lodash')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

const unsubscribeSlug = config.get('unsubscribe.url')

module.exports = {
  async sendNotifications({
    baseUrl,
    fragment,
    UserModel,
    collection,
    targetUserName,
    isEditorInChief,
    newRecommendation,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const fragmentAuthors = await fragmentHelper.getAuthorData({ UserModel })

    const subjectBaseText = `${collection.customId}: Manuscript`
    const titleText = `the manuscript titled "${parsedFragment.title}" by ${
      fragmentAuthors.submittingAuthor.firstName
    } ${fragmentAuthors.submittingAuthor.lastName}`

    const userHelper = new User({ UserModel })
    const eicName = await userHelper.getEiCName()

    const email = new Email({
      type: 'user',
      content: {
        unsubscribeLink: baseUrl,
        ctaText: 'MANUSCRIPT DETAILS',
        signatureName: eicName,
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
      },
    })

    let comments
    if (isEditorInChief) {
      const eicComments = chain(newRecommendation)
        .get('comments')
        .find(comm => !comm.public)
        .get('content')
        .value()

      comments = eicComments
    }

    const hasPeerReview = (collection = {}) =>
      !isEmpty(collection.handlingEditor)

    if (
      (isEditorInChief || newRecommendation.recommendationType === 'review') &&
      hasPeerReview(collection)
    ) {
      // the request came from either the Editor in Chief or a reviewer, so the HE needs to be notified
      sendHandlingEditorEmail({
        email,
        eicName: await userHelper.getEiCName(),
        baseUrl,
        comments,
        titleText,
        targetUserName,
        subjectBaseText,
        handlingEditor: get(collection, 'handlingEditor', {}),
        recommendation: newRecommendation.recommendation,
        recommendationType: newRecommendation.recommendationType,
      })
    }

    if (
      newRecommendation.recommendationType !== 'review' &&
      newRecommendation.recommendation !== 'return-to-handling-editor'
    ) {
      if (isEditorInChief || collection.status === 'revisionRequested') {
        sendAuthorsEmail({
          email,
          baseUrl,
          titleText,
          parsedFragment,
          fragmentAuthors,
          isEditorInChief,
          subjectBaseText,
          newRecommendation,
          handlingEditorName: get(collection, 'handlingEditor.name', eicName),
        })
      }
      if (hasPeerReview(collection)) {
        sendReviewersEmail({
          email,
          baseUrl,
          UserModel,
          titleText,
          fragmentHelper,
          isEditorInChief,
          subjectBaseText,
          recommendation: newRecommendation.recommendation,
          handlingEditorName: get(collection, 'handlingEditor.name', 'N/A'),
        })

        sendEiCsEmail({
          email,
          baseUrl,
          userHelper,
          titleText,
          subjectBaseText,
          recommendation: newRecommendation,
        })
      }
    }
  },
}

const sendHandlingEditorEmail = ({
  email,
  eicName,
  baseUrl,
  comments,
  titleText,
  handlingEditor,
  targetUserName,
  recommendation,
  subjectBaseText,
  recommendationType,
}) => {
  let emailType
  if (recommendationType === 'review') {
    email.content.subject = `${subjectBaseText} Review`
    emailType = 'he-review-submitted'
  } else {
    email.content.subject = `${subjectBaseText} Decision`
    switch (recommendation) {
      case 'return-to-handling-editor':
        emailType = 'he-manuscript-return-with-comments'
        break
      case 'publish':
        emailType = 'he-manuscript-published'
        break
      case 'reject':
        emailType = 'he-manuscript-rejected'
        break
      default:
        throw new Error(`undefined recommendation: ${recommendation} `)
    }
  }

  email.toUser = {
    email: handlingEditor.email,
    name: handlingEditor.name,
  }
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
      comments,
      targetUserName,
    }),
  })
  email.sendEmail({ html, text })
}

const sendAuthorsEmail = async ({
  email,
  baseUrl,
  titleText,
  isEditorInChief,
  subjectBaseText,
  fragmentAuthors,
  newRecommendation,
  handlingEditorName,
  parsedFragment: { heRecommendation },
}) => {
  let emailType, authors, comments

  if (isEditorInChief) {
    const heComments = get(heRecommendation, 'comments', [])
    if (heComments.length) {
      const publicComment = heComments.find(comm => comm.public)
      const content = get(publicComment, 'content')
      if (!content) {
        throw new Error('a public comment cannot be without content')
      }
      comments = `Reason & Details: "${content}"`
    }

    if (newRecommendation.recommendation === 'publish') {
      emailType = 'author-manuscript-published'
      email.content.subject = `${subjectBaseText} Published`
    } else {
      emailType = 'author-manuscript-rejected'
      email.content.subject = `${subjectBaseText} Decision`
    }

    authors = fragmentAuthors.activeAuthors.map(author => ({
      ...author,
      ...getEmailCopy({
        emailType,
        titleText,
        comments,
      }),
    }))
  } else {
    emailType = 'author-request-to-revision'

    email.content.subject = `${subjectBaseText} Recommendation`
    email.content.signatureName = handlingEditorName

    const authorNote = newRecommendation.comments.find(comm => comm.public)
    const content = get(authorNote, 'content')
    const authorNoteText = content ? `Reason & Details: "${content}"` : ''
    authors = [
      {
        ...fragmentAuthors.submittingAuthor,
        ...getEmailCopy({
          emailType,
          titleText,
          comments: authorNoteText,
        }),
      },
    ]
  }

  authors.forEach(author => {
    email.toUser = {
      email: author.email,
      name: `${author.firstName} ${author.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: author.id,
      },
    )
    const { html, text } = email.getBody({
      body: {
        paragraph: author.paragraph,
        hasLink: author.hasLink,
      },
    })
    email.sendEmail({ html, text })
  })
}

const sendReviewersEmail = async ({
  email,
  baseUrl,
  eicName,
  titleText,
  UserModel,
  fragmentHelper,
  recommendation,
  isEditorInChief,
  subjectBaseText,
  handlingEditorName,
}) => {
  let reviewers
  if (isEditorInChief) {
    email.content.subject = `${subjectBaseText} Decision`
    const emailType =
      recommendation === 'publish'
        ? 'submitted-reviewers-after-publish'
        : 'submitted-reviewers-after-reject'

    reviewers = (await fragmentHelper.getReviewers({
      UserModel,
      type: 'submitted',
    })).map(rev => ({
      ...rev,
      emailType,
      ...getEmailCopy({
        emailType,
        titleText,
      }),
    }))
  } else {
    email.content.signatureName = handlingEditorName
    email.content.subject = `${subjectBaseText} ${getSubjectByRecommendation(
      recommendation,
    )}`

    const acceptedReviewers = (await fragmentHelper.getReviewers({
      UserModel,
      type: 'accepted',
    })).map(rev => ({
      ...rev,
      ...getEmailCopy({
        emailType: 'accepted-reviewers-after-recommendation',
        titleText,
      }),
    }))

    const pendingReviewers = (await fragmentHelper.getReviewers({
      UserModel,
      type: 'pending',
    })).map(rev => ({
      ...rev,
      ...getEmailCopy({
        emailType: 'pending-reviewers-after-recommendation',
        titleText,
      }),
    }))

    reviewers = [...acceptedReviewers, ...pendingReviewers]
  }

  reviewers.forEach(reviewer => {
    email.toUser = {
      email: reviewer.email,
      name: `${reviewer.firstName} ${reviewer.lastName}`,
    }
    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: reviewer.id,
      },
    )
    const { html, text } = email.getBody({
      body: { paragraph: reviewer.paragraph, hasLink: reviewer.hasLink },
    })
    email.sendEmail({ html, text })
  })
}

const sendEiCsEmail = async ({
  email,
  titleText,
  userHelper,
  subjectBaseText,
  recommendation: { recommendation, comments: recComments = [] },
}) => {
  let emailType

  email.content.subject = `${subjectBaseText} Recommendation`
  switch (recommendation) {
    case 'minor':
    case 'major':
      emailType = 'eic-request-revision-from-he'
      break
    case 'publish':
      emailType = 'eic-recommend-to-publish-from-he'
      break
    case 'reject':
      emailType = 'eic-recommend-to-reject-from-he'
      break
    default:
      throw new Error(`undefined recommendation: ${recommendation} `)
  }

  const privateNote = recComments.find(comm => !comm.public)
  const content = get(privateNote, 'content')
  const comments = content ? `Note to Editor: "${content}"` : ''

  const editors = (await userHelper.getEditorsInChief()).map(eic => ({
    ...eic,
    ...getEmailCopy({
      emailType,
      titleText,
      comments,
    }),
  }))

  editors.forEach(eic => {
    email.toUser = {
      email: eic.email,
      name: `${eic.firstName} ${eic.lastName}`,
    }
    const { html, text } = email.getBody({
      body: { paragraph: eic.paragraph, hasLink: eic.hasLink },
    })
    email.sendEmail({ html, text })
  })
}

const getSubjectByRecommendation = recommendation =>
  ['minor', 'major'].includes(recommendation)
    ? 'Revision Requested'
    : 'Recommendation Submitted'
